@@ -27,6 +27,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include "glpk.h"
+#include "glpmpl.h"
 
 /*--------------------------------------------------------------------*/
 /* This program is a stand-alone LP/MIP solver. For pure LP problems  */
@@ -35,14 +36,16 @@
 /* on the simplex method is used.                                     */
 /*--------------------------------------------------------------------*/
 
-static char *version = "GLPSOL -- GLPK LP/MIP Solver, Version 4.4";
+static char *version = "GLPSOL -- GLPK LP/MIP Solver, Version 4.7";
 /* version string */
 
 static int format = 0;
 /* type of input text file:
-   0 - MPS
+   0 - fixed MPS
    1 - CPLEX LP
-   2 - GNU MathProg */
+   2 - GNU MathProg
+   3 - GNU LP
+   4 - free MPS */
 
 static char *in_file = NULL;
 /* name of input text file */
@@ -55,6 +58,14 @@ static char *display = NULL;
 /* name of optional output text file, to which display output is sent;
    NULL means the output is sent to stdout */
 
+static char *in_bas = NULL;
+/* name of input text file, which contains initial LP basis in MPS
+   format; NULL means no initial LP basis is provided */
+
+static char *out_bas = NULL;
+/* name of output text file, to which final LP basis should be written
+   in MPS format; NULL means no output */
+
 static int dir = 0;
 /* optimization direction flag:
    0       - not specified
@@ -94,9 +105,13 @@ static int orig = 0;
 
 static char *out_mps = NULL;
 /* name of output text file, to which the problem should be written in
-   MPS format; NULL means no MPS output */
+   fixed MPS format; NULL means no MPS output */
 
-static char *out_lpt = NULL;
+static char *out_freemps = NULL;
+/* name of output text file, to which the problem should be written in
+   free MPS format; NULL means no MPS output */
+
+static char *out_cpxlp = NULL;
 /* name of output text file, to which the problem should be written in
    CPLEX LP format; NULL means no CPLEX LP output */
 
@@ -104,6 +119,13 @@ static char *out_txt = NULL;
 /* name of output text file, to which the problem should be written in
    plain text format; NULL means no plain text output */
 
+static char *out_glp = NULL;
+/* name of output text file, to which the problem should be written in
+   GNU LP format; NULL means no GNU LP output */
+
+static char *newname = NULL;
+/* new name which has to be assigned to the problem */
+
 static int basis = 1;
 /* which initial basis should be used:
    0 - standard initial basis
@@ -129,15 +151,17 @@ static int nomip = 0;
 
 static int branch = 2;
 /* which branching technique should be used:
-   0 - on the first integer variable
-   1 - on the last integer variable
-   2 - using heuristic by Driebeek and Tomlin */
+   0 - branch on first variable
+   1 - branch on last variable
+   2 - branch using heuristic by Driebeck and Tomlin
+   3 - branch on most fractional variable */
 
 static int btrack = 2;
 /* which backtracking technique should be used:
-   0 - depth first search
-   1 - breadth first search
-   2 - the best projection heuristic */
+   0 - select most recent node (depth first search)
+   1 - select earliest node (breadth first search)
+   2 - select node using the best projection heuristic
+   3 - select node with best local bound */
 
 /*----------------------------------------------------------------------
 -- display_help - display help.
@@ -149,9 +173,12 @@ static void display_help(char *my_name)
 {     print("Usage: %s [options...] filename", my_name);
       print("");
       print("General options:");
-      print("   --mps             read LP/MIP problem in MPS format (de"
-         "fault)");
-      print("   --lpt             read LP/MIP problem in CPLEX LP forma"
+      print("   --glp             read LP/MIP model in GNU LP format");
+      print("   --mps             read LP/MIP problem in fixed MPS form"
+         "at (default)");
+      print("   --freemps         read LP/MIP problem in free MPS forma"
+         "t");
+      print("   --cpxlp           read LP/MIP problem in CPLEX LP forma"
          "t");
       print("   --math            read LP/MIP model written in GNU Math"
          "Prog modeling");
@@ -188,13 +215,20 @@ static void display_help(char *my_name)
       print("   --tmlim nnn       limit solution time to nnn seconds");
       print("   --check           do not solve problem, check input dat"
          "a only");
+      print("   --name probname   change problem name to probname");
       print("   --plain           use plain names of rows and columns ("
          "default)");
       print("   --orig            try using original names of rows and "
          "columns");
-      print("   --wmps filename   write problem to filename in MPS form"
-         "at");
-      print("   --wlpt filename   write problem to filename in CPLEX LP"
+      print("                     (default for --mps)");
+      print("   --wglp filename   write problem to filename in GNU LP f"
+         "ormat");
+      print("   --wmps filename   write problem to filename in fixed MP"
+         "S format");
+      print("   --wfreemps filename");
+      print("                     write problem to filename in free MPS"
+         " format");
+      print("   --wcpxlp filename write problem to filename in CPLEX LP"
          " format");
       print("   --wtxt filename   write problem to filename in plain te"
          "xt format");
@@ -207,6 +241,8 @@ static void display_help(char *my_name)
          "cks");
       print("   --adv             use advanced initial basis (default)")
          ;
+      print("   --bas filename    read initial basis from filename in M"
+         "PS format");
       print("   --steep           use steepest edge technique (default)"
          );
       print("   --nosteep         use standard \"textbook\" pricing");
@@ -217,6 +253,8 @@ static void display_help(char *my_name)
       print("   --presol          use presolver (default; assumes --sca"
          "le and --adv)");
       print("   --nopresol        do not use presolver");
+      print("   --wbas filename   write final basis to filename in MPS "
+         "format");
       print("");
       print("Options specific to MIP:");
       print("   --nomip           consider all integer variables as con"
@@ -227,12 +265,15 @@ static void display_help(char *my_name)
       print("   --drtom           branch using heuristic by Driebeck an"
          "d Tomlin");
       print("                     (default)");
+      print("   --mostf           branch on most fractional varaible");
       print("   --dfs             backtrack using depth first search");
       print("   --bfs             backtrack using breadth first search")
          ;
       print("   --bestp           backtrack using the best projection h"
          "euristic");
       print("                     (default)");
+      print("   --bestb           backtrack using node with best local "
+         "bound");
       print("");
       print("For description of the MPS and CPLEX LP formats see Refere"
          "nce Manual.");
@@ -279,17 +320,21 @@ static void parse_cmdline(int argc, char *argv[])
       for (k = 1; k < argc; k++)
       {  if (p("--mps"))
             format = 0;
-         else if (p("--lpt"))
+         else if (p("--cpxlp") || p("--lpt"))
             format = 1;
          else if (p("--math") || p("-m") || p("--model"))
             format = 2;
+         else if (p("--glp"))
+            format = 3;
+         else if (p("--freemps"))
+            format = 4;
          else if (p("-d") || p("--data"))
          {  k++;
             if (k == argc || argv[k][0] == '\0' || argv[k][0] == '-')
             {  print("No data input file specifed");
                exit(EXIT_FAILURE);
             }
-            if(in_data != NULL)
+            if (in_data != NULL)
             {  print("Only one data input file allowed");
                exit(EXIT_FAILURE);
             }
@@ -358,6 +403,18 @@ static void parse_cmdline(int argc, char *argv[])
          }
          else if (p("--check"))
             check = 1;
+         else if (p("--name"))
+         {  k++;
+            if (k == argc || argv[k][0] == '\0' || argv[k][0] == '-')
+            {  print("No problem name specified");
+               exit(EXIT_FAILURE);
+            }
+            if (newname != NULL)
+            {  print("Only one problem name allowed");
+               exit(EXIT_FAILURE);
+            }
+            newname = argv[k];
+         }
          else if (p("--plain"))
             orig = 0;
          else if (p("--orig"))
@@ -365,26 +422,38 @@ static void parse_cmdline(int argc, char *argv[])
          else if (p("--wmps"))
          {  k++;
             if (k == argc || argv[k][0] == '\0' || argv[k][0] == '-')
-            {  print("No MPS output file specified");
+            {  print("No fixed MPS output file specified");
                exit(EXIT_FAILURE);
             }
             if (out_mps != NULL)
-            {  print("Only one MPS output file allowed");
+            {  print("Only one fixed MPS output file allowed");
                exit(EXIT_FAILURE);
             }
             out_mps = argv[k];
          }
-         else if (p("--wlpt"))
+         else if (p("--wfreemps"))
+         {  k++;
+            if (k == argc || argv[k][0] == '\0' || argv[k][0] == '-')
+            {  print("No free MPS output file specified");
+               exit(EXIT_FAILURE);
+            }
+            if (out_mps != NULL)
+            {  print("Only one free MPS output file allowed");
+               exit(EXIT_FAILURE);
+            }
+            out_freemps = argv[k];
+         }
+         else if (p("--wcpxlp") || p("--wlpt"))
          {  k++;
             if (k == argc || argv[k][0] == '\0' || argv[k][0] == '-')
             {  print("No CPLEX LP output file specified");
                exit(EXIT_FAILURE);
             }
-            if (out_lpt != NULL)
+            if (out_cpxlp != NULL)
             {  print("Only one CPLEX LP output file allowed");
                exit(EXIT_FAILURE);
             }
-            out_lpt = argv[k];
+            out_cpxlp = argv[k];
          }
          else if (p("--wtxt"))
          {  k++;
@@ -398,6 +467,18 @@ static void parse_cmdline(int argc, char *argv[])
             }
             out_txt = argv[k];
          }
+         else if (p("--wglp"))
+         {  k++;
+            if (k == argc || argv[k][0] == '\0' || argv[k][0] == '-')
+            {  print("No problem output file specified");
+               exit(EXIT_FAILURE);
+            }
+            if (out_glp != NULL)
+            {  print("Only one problem output file allowed");
+               exit(EXIT_FAILURE);
+            }
+            out_glp = argv[k];
+         }
          else if (p("-h") || p("--help"))
             display_help(argv[0]);
          else if (p("-v") || p("--version"))
@@ -406,6 +487,18 @@ static void parse_cmdline(int argc, char *argv[])
             basis = 0;
          else if (p("--adv"))
             basis = 1;
+         else if (p("--bas"))
+         {  k++;
+            if (k == argc || argv[k][0] == '\0' || argv[k][0] == '-')
+            {  print("No basis input file specifed");
+               exit(EXIT_FAILURE);
+            }
+            if (in_bas != NULL)
+            {  print("Only one basis input file allowed");
+               exit(EXIT_FAILURE);
+            }
+            in_bas = argv[k];
+         }
          else if (p("--steep"))
             price = 1;
          else if (p("--nosteep"))
@@ -418,6 +511,18 @@ static void parse_cmdline(int argc, char *argv[])
             presol = 1;
          else if (p("--nopresol"))
             presol = 0;
+         else if (p("--wbas"))
+         {  k++;
+            if (k == argc || argv[k][0] == '\0' || argv[k][0] == '-')
+            {  print("No basis output file specified");
+               exit(EXIT_FAILURE);
+            }
+            if (out_bas != NULL)
+            {  print("Only one basis output file allowed");
+               exit(EXIT_FAILURE);
+            }
+            out_bas = argv[k];
+         }
          else if (p("--nomip"))
             nomip = 1;
          else if (p("--first"))
@@ -426,12 +531,16 @@ static void parse_cmdline(int argc, char *argv[])
             branch = 1;
          else if (p("--drtom"))
             branch = 2;
+         else if (p("--mostf"))
+            branch = 3;
          else if (p("--dfs"))
             btrack = 0;
          else if (p("--bfs"))
             btrack = 1;
          else if (p("--bestp"))
             btrack = 2;
+         else if (p("--bestb"))
+            btrack = 3;
          else if (argv[k][0] == '-' ||
                  (argv[k][0] == '-' && argv[k][1] == '-'))
          {  print("Invalid option `%s'; try %s --help",
@@ -457,18 +566,22 @@ static void parse_cmdline(int argc, char *argv[])
 
 int main(int argc, char *argv[])
 {     LPX *lp;
+      MPL *mpl = NULL;
       int ret;
       double start;
       /* parse command line parameters */
       parse_cmdline(argc, argv);
       /* remove all output files specified in the command line */
       if (display != NULL) remove(display);
+      if (out_bas != NULL) remove(out_bas);
       if (out_sol != NULL) remove(out_sol);
       if (out_bnds != NULL) remove(out_bnds);
       if (out_mps != NULL) remove(out_mps);
-      if (out_lpt != NULL) remove(out_lpt);
+      if (out_freemps != NULL) remove(out_freemps);
+      if (out_cpxlp != NULL) remove(out_cpxlp);
       if (out_txt != NULL) remove(out_txt);
-      /* read problem from the input file */
+      if (out_glp != NULL) remove(out_glp);
+      /* read problem data from the input file */
       if (in_file == NULL)
       {  print("No input file specified; try %s --help", argv[0]);
          exit(EXIT_FAILURE);
@@ -480,20 +593,46 @@ int main(int argc, char *argv[])
             {  print("MPS file processing error");
                exit(EXIT_FAILURE);
             }
+            orig = 1;
             break;
          case 1:
-            lp = lpx_read_lpt(in_file);
+            lp = lpx_read_cpxlp(in_file);
             if (lp == NULL)
             {  print("CPLEX LP file processing error");
                exit(EXIT_FAILURE);
             }
             break;
          case 2:
+#if 0 /* 01/VIII-2004 */
             lp = lpx_read_model(in_file, in_data, display);
             if (lp == NULL)
             {  print("Model processing error");
                exit(EXIT_FAILURE);
             }
+#else
+            /* initialize the translator database */
+            mpl = mpl_initialize();
+            /* read model section and optional data section */
+            ret = mpl_read_model(mpl, in_file, in_data != NULL);
+            if (ret == 4)
+err:        {  print("Model processing error");
+               exit(EXIT_FAILURE);
+            }
+            insist(ret == 1 || ret == 2);
+            /* read data section, if necessary */
+            if (in_data != NULL)
+            {  insist(ret == 1);
+               ret = mpl_read_data(mpl, in_data);
+               if (ret == 4) goto err;
+               insist(ret == 2);
+            }
+            /* generate model */
+            ret = mpl_generate(mpl, display);
+            if (ret == 4) goto err;
+            /* extract problem instance */
+            lp = lpx_extract_prob(mpl);
+            insist(lp != NULL);
+#endif
             if (lpx_get_num_rows(lp) == 0)
             {  print("Problem has no rows");
                exit(EXIT_FAILURE);
@@ -503,24 +642,49 @@ int main(int argc, char *argv[])
                exit(EXIT_FAILURE);
             }
             break;
+         case 3:
+            lp = lpx_read_prob(in_file);
+            if (lp == NULL)
+            {  print("GNU LP file processing error");
+               exit(EXIT_FAILURE);
+            }
+            break;
+         case 4:
+            lp = lpx_read_freemps(in_file);
+            if (lp == NULL)
+            {  print("MPS file processing error");
+               exit(EXIT_FAILURE);
+            }
+            break;
          default:
             insist(format != format);
       }
+      /* order rows and columns of the constraint matrix */
+      lpx_order_matrix(lp);
+      /* change problem name (if required) */
+      if (newname != NULL) lpx_set_prob_name(lp, newname);
       /* change optimization direction (if required) */
       if (dir != 0) lpx_set_obj_dir(lp, dir);
-      /* write problem in MPS format (if required) */
+      /* write problem in fixed MPS format (if required) */
       if (out_mps != NULL)
       {  lpx_set_int_parm(lp, LPX_K_MPSORIG, orig);
          ret = lpx_write_mps(lp, out_mps);
          if (ret != 0)
-         {  print("Unable to write problem in MPS format");
+         {  print("Unable to write problem in fixed MPS format");
+            exit(EXIT_FAILURE);
+         }
+      }
+      /* write problem in free MPS format (if required) */
+      if (out_freemps != NULL)
+      {  ret = lpx_write_freemps(lp, out_freemps);
+         if (ret != 0)
+         {  print("Unable to write problem in free MPS format");
             exit(EXIT_FAILURE);
          }
       }
       /* write problem in CPLEX LP format (if required) */
-      if (out_lpt != NULL)
-      {  lpx_set_int_parm(lp, LPX_K_LPTORIG, orig);
-         ret = lpx_write_lpt(lp, out_lpt);
+      if (out_cpxlp != NULL)
+      {  ret = lpx_write_cpxlp(lp, out_cpxlp);
          if (ret != 0)
          {  print("Unable to write problem in CPLEX LP format");
             exit(EXIT_FAILURE);
@@ -535,12 +699,41 @@ int main(int argc, char *argv[])
             exit(EXIT_FAILURE);
          }
       }
+      /* write problem in GNU LP format (if required) */
+      if (out_glp != NULL)
+      {  ret = lpx_write_prob(lp, out_glp);
+         if (ret != 0)
+         {  print("Unable to write problem in GNU LP format");
+            exit(EXIT_FAILURE);
+         }
+      }
       /* if only data check is required, skip computations */
       if (check) goto skip;
       /* scale the problem data (if required) */
       if (scale && (!presol || method == 1)) lpx_scale_prob(lp);
       /* build advanced initial basis (if required) */
-      if (method == 0 && basis && !presol) lpx_adv_basis(lp);
+      if (method == 0 && basis && !presol && in_bas == NULL)
+         lpx_adv_basis(lp);
+      /* or read initial basis from input text file in MPS format */
+      if (in_bas != NULL)
+      {  if (method != 0)
+         {  print("Initial LP basis is useless for interior-point solve"
+               "r and therefore ignored");
+            goto nobs;
+         }
+         lpx_set_int_parm(lp, LPX_K_MPSORIG, orig);
+         ret = lpx_read_bas(lp, in_bas);
+         if (ret != 0)
+         {  print("Unable to read initial LP basis");
+            exit(EXIT_FAILURE);
+         }
+         if (presol)
+         {  presol = 0;
+            print("LP presolver disabled because initial LP basis has b"
+               "een provided");
+         }
+nobs:    ;
+      }
       /* set some control parameters, which might be changed in the
          command line */
       lpx_set_int_parm(lp, LPX_K_PRICE, price);
@@ -555,7 +748,8 @@ int main(int argc, char *argv[])
       {  case 0:
             if (nomip || lpx_get_class(lp) == LPX_LP)
             {  ret = lpx_simplex(lp);
-               if (presol && ret != LPX_E_OK && out_sol != NULL)
+               if (presol && ret != LPX_E_OK && (out_bas != NULL ||
+                  out_sol != NULL))
                   print("If you need actual output for non-optimal solu"
                      "tion, use --nopresol");
             }
@@ -572,7 +766,7 @@ int main(int argc, char *argv[])
             if (nomip || lpx_get_class(lp) == LPX_LP)
                lpx_interior(lp);
             else
-            {  print("Interior point method is not able to solve MIP pr"
+            {  print("Interior-point method is not able to solve MIP pr"
                   "oblem; use --simplex");
                exit(EXIT_FAILURE);
             }
@@ -585,6 +779,48 @@ int main(int argc, char *argv[])
       print("Memory used: %.1fM (%d bytes)",
          (double)lib_env_ptr()->mem_tpeak / (double)(1024 * 1024),
          lib_env_ptr()->mem_tpeak);
+#if 1 /* 01/VIII-2004 */
+      if (mpl != NULL && mpl_has_solve_stmt(mpl))
+      {  int n, j, round;
+         /* store the solution to the translator database */
+         n = lpx_get_num_cols(lp);
+         round = lpx_get_int_parm(lp, LPX_K_ROUND);
+         lpx_set_int_parm(lp, LPX_K_ROUND, 1);
+         switch (method)
+         {  case 0:
+               for (j = 1; j <= n; j++)
+                  mpl_put_col_value(mpl, j, lpx_get_col_prim(lp, j));
+               break;
+            case 1:
+               for (j = 1; j <= n; j++)
+                  mpl_put_col_value(mpl, j, lpx_ipt_col_prim(lp, j));
+               break;
+            case 2:
+               for (j = 1; j <= n; j++)
+                  mpl_put_col_value(mpl, j, lpx_mip_col_val(lp, j));
+               break;
+            default:
+               insist(method != method);
+         }
+         lpx_set_int_parm(lp, LPX_K_ROUND, round);
+         /* perform postsolving */
+         ret = mpl_postsolve(mpl);
+         if (ret == 4)
+         {  print("Model postsolving error");
+            exit(EXIT_FAILURE);
+         }
+         insist(ret == 3);
+      }
+#endif
+      /* write final LP basis (if required) */
+      if (out_bas != NULL)
+      {  lpx_set_int_parm(lp, LPX_K_MPSORIG, orig);
+         ret = lpx_write_bas(lp, out_bas);
+         if (ret != 0)
+         {  print("Unable to write final LP basis");
+            exit(EXIT_FAILURE);
+         }
+      }
       /* write problem solution found by the solver (if required) */
       if (out_sol != NULL)
       {  switch (method)
@@ -620,6 +856,11 @@ int main(int argc, char *argv[])
       }
 skip: /* delete the problem object */
       lpx_delete_prob(lp);
+#if 1 /* 01/VIII-2004 */
+      /* if the translator database exists, destroy it */
+      if (mpl != NULL) mpl_terminate(mpl);
+#endif
+      /* check that no memory blocks are still allocated */
       insist(lib_env_ptr()->mem_total == 0);
       insist(lib_env_ptr()->mem_count == 0);
       /* return to the control program */
