@@ -57,6 +57,9 @@
 -- 6. Run Concorde to solve your TSP instances. See also for TSPLIB 95,
 --    a library of standard TSP instances; this library is available at
 --    http://www.iwr.uni-heidelberg.de/groups/comopt/software/TSPLIB95/
+--    (some results given in the report tsplib95.ps are wrong; for more
+--    correct results see the webpage .../TSPLIB95/stsp.html on the site
+--    above).
 --
 -- Note that these interface routines do not include many important
 -- features (for example, efficient strong branching), therefore hard
