@@ -1,11 +1,11 @@
-/* glpipm.h */
+/* glpipm.h (primal-dual interior point method) */
 
 /*----------------------------------------------------------------------
--- Copyright (C) 2000, 2001, 2002 Andrew Makhorin <mao@mai2.rcnet.ru>,
---               Department for Applied Informatics, Moscow Aviation
---               Institute, Moscow, Russia. All rights reserved.
+-- Copyright (C) 2000, 2001, 2002, 2003, 2004 Andrew Makhorin,
+-- Department for Applied Informatics, Moscow Aviation Institute,
+-- Moscow, Russia. All rights reserved. E-mail: <mao@mai2.rcnet.ru>.
 --
--- This file is a part of GLPK (GNU Linear Programming Kit).
+-- This file is part of GLPK (GNU Linear Programming Kit).
 --
 -- GLPK is free software; you can redistribute it and/or modify it
 -- under the terms of the GNU General Public License as published by
@@ -26,7 +26,7 @@
 #ifndef _GLPIPM_H
 #define _GLPIPM_H
 
-#include "glplp.h"
+#include "glpmat.h"
 
 #define ipm1                  glp_ipm1
 #define ipm1_driver           glp_ipm1_driver
@@ -35,11 +35,6 @@ extern int ipm1(MAT *A, double b[], double c[], double x[], double y[],
       double z[]);
 /* primal-dual interior point method for linear programming */
 
-#if 0
-extern int ipm1_driver(LP *lp, LPSOL *sol);
-/* driver for primal-dual interior point method */
-#endif
-
 #endif
 
 /* eof */
