@@ -47,9 +47,6 @@
 #define strspx                glp_lib_strspx
 #define strtrim               glp_lib_strtrim
 #define fp2rat                glp_lib_fp2rat
-#define lib_init_rand         glp_lib_init_rand
-#define lib_next_rand         glp_lib_next_rand
-#define lib_unif_rand         glp_lib_unif_rand
 
 typedef struct LIBENV LIBENV;
 typedef struct LIBMEM LIBMEM;
@@ -90,12 +87,6 @@ struct LIBENV
       void *file_slot[LIB_MAX_OPEN]; /* FILE *file_slot[]; */
       /* file_slot[k], 0 <= k <= LIB_MAX_OPEN-1, is a pointer to k-th
          i/o stream; if k-th slot is free, file_slot[k] is NULL */
-      /*--------------------------------------------------------------*/
-      /* pseudo-random number generator */
-      int rand_val[56];
-      /* pseudo-random values */
-      int *next_val;
-      /* the next pseudo-random value to be exported */
 };
 
 struct LIBMEM
@@ -194,15 +185,6 @@ char *strtrim(char *str);
 int fp2rat(double x, double eps, double *p, double *q);
 /* convert floating-point number to rational number */
 
-void lib_init_rand(int seed);
-/* initialize pseudo-random number generator */
-
-int lib_next_rand(void);
-/* obtain pseudo-random integer in [0, 2^31-1] */
-
-int lib_unif_rand(int m);
-/* obtain pseudo-random integer in [0, m-1] */
-
 #endif
 
 /* eof */
