@@ -26,6 +26,7 @@
 #ifndef _GLPLPX_H
 #define _GLPLPX_H
 
+#include "glpavl.h"
 #include "glpinv.h"
 #include "glpstr.h"
 
@@ -45,6 +46,7 @@
 #define lpx_set_mat_row       glp_lpx_set_mat_row
 #define lpx_set_mat_col       glp_lpx_set_mat_col
 #define lpx_load_matrix       glp_lpx_load_matrix
+#define lpx_order_matrix      glp_lpx_order_matrix
 #define lpx_set_rii           glp_lpx_set_rii
 #define lpx_set_sjj           glp_lpx_set_sjj
 #define lpx_set_row_stat      glp_lpx_set_row_stat
@@ -52,6 +54,12 @@
 #define lpx_del_rows          glp_lpx_del_rows
 #define lpx_del_cols          glp_lpx_del_cols
 #define lpx_delete_prob       glp_lpx_delete_prob
+#if 1 /* 15/VIII-2004 */
+#define lpx_create_index      glp_lpx_create_index
+#define lpx_find_row          glp_lpx_find_row
+#define lpx_find_col          glp_lpx_find_col
+#define lpx_delete_index      glp_lpx_delete_index
+#endif
 #define lpx_put_lp_basis      glp_lpx_put_lp_basis
 #define lpx_put_solution      glp_lpx_put_solution
 #define lpx_put_ray_info      glp_lpx_put_ray_info
@@ -171,14 +179,19 @@
 #define lpx_write_mps         glp_lpx_write_mps
 #define lpx_read_bas          glp_lpx_read_bas
 #define lpx_write_bas         glp_lpx_write_bas
+#define lpx_read_freemps      glp_lpx_read_freemps
+#define lpx_write_freemps     glp_lpx_write_freemps
 #define lpx_print_prob        glp_lpx_print_prob
 #define lpx_print_sol         glp_lpx_print_sol
 #define lpx_print_ips         glp_lpx_print_ips
 #define lpx_print_mip         glp_lpx_print_mip
 #define lpx_print_sens_bnds   glp_lpx_print_sens_bnds
-#define lpx_read_lpt          glp_lpx_read_lpt
-#define lpx_write_lpt         glp_lpx_write_lpt
+#define lpx_read_cpxlp        glp_lpx_read_cpxlp
+#define lpx_write_cpxlp       glp_lpx_write_cpxlp
+#define lpx_extract_prob      glp_lpx_extract_prob
 #define lpx_read_model        glp_lpx_read_model
+#define lpx_read_prob         glp_lpx_read_prob
+#define lpx_write_prob        glp_lpx_write_prob
 
 /*----------------------------------------------------------------------
 -- The structure LPX is an LP/MIP problem object, which corresponds to
@@ -373,6 +386,14 @@ struct LPX
       LPXCOL **col; /* LPXCOL *col[1+n_max]; */
       /* col[0] is not used;
          col[j], 1 <= j <= n, is a pointer to j-th column */
+#if 1 /* 15/VIII-2004 */
+      AVLTREE *r_tree;
+      /* row index to find rows by their names; NULL means this index
+         does not exist */
+      AVLTREE *c_tree;
+      /* column index to find columns by their names; NULL means this
+         index does not exist */
+#endif
       /*--------------------------------------------------------------*/
       /* LP basis */
       int b_stat;
@@ -495,14 +516,16 @@ struct LPX
          to the standard output; zero value means no delay */
       int branch; /* MIP */
       /* branching heuristic:
-         0 - branch on the first variable
-         1 - branch on the last variable
-         2 - branch using a heuristic by Driebeck and Tomlin */
+         0 - branch on first variable
+         1 - branch on last variable
+         2 - branch using heuristic by Driebeck and Tomlin
+         3 - branch on most fractional variable */
       int btrack; /* MIP */
       /* backtracking heuristic:
-         0 - depth first search
-         1 - breadth first search
-         2 - backtrack using the best projection heuristic */
+         0 - select most recent node (depth first search)
+         1 - select earliest node (breadth first search)
+         2 - select node using the best projection heuristic
+         3 - select node with best local bound */
       double tol_int; /* MIP */
       /* absolute tolerance used to check if the current basic solution
          is integer feasible */
@@ -553,6 +576,12 @@ struct LPXROW
       STR *name;
       /* row name (1 to 255 chars); NULL means no name is assigned to
          this row */
+#if 1 /* 15/VIII-2004 */
+      AVLNODE *node;
+      /* pointer to corresponding node in the row index; NULL means
+         that either the row index does not exist or this row has no
+         name assigned */
+#endif
       int type;
       /* type of the auxiliary variable: */
 #define LPX_FR          110   /* free variable */
@@ -602,6 +631,12 @@ struct LPXCOL
       STR *name;
       /* column name (1 to 255 chars); NULL means no name is assigned
          to this column */
+#if 1 /* 15/VIII-2004 */
+      AVLNODE *node;
+      /* pointer to corresponding node in the column index; NULL means
+         that either the column index does not exist or the column has
+         no name assigned */
+#endif
       int kind;
       /* kind of the structural variable: */
 #define LPX_CV          160   /* continuous variable */
@@ -861,6 +896,9 @@ void lpx_set_mat_col(LPX *lp, int j, int len, int ind[], double val[]);
 void lpx_load_matrix(LPX *lp, int ne, int ia[], int ja[], double ar[]);
 /* load (replace) the whole constraint matrix */
 
+void lpx_order_matrix(LPX *lp);
+/* order rows and columns of the constraint matrix */
+
 void lpx_set_rii(LPX *lp, int i, double rii);
 /* set (change) row scale factor */
 
@@ -882,6 +920,13 @@ void lpx_del_cols(LPX *lp, int ncs, int num[]);
 void lpx_delete_prob(LPX *lp);
 /* delete problem object */
 
+#if 1 /* 15/VIII-2004 */
+void lpx_create_index(LPX *lp);
+int lpx_find_row(LPX *lp, char *name);
+int lpx_find_col(LPX *lp, char *name);
+void lpx_delete_index(LPX *lp);
+#endif
+
 void lpx_put_lp_basis(LPX *lp, int b_stat, int basis[], INV *b_inv);
 /* store LP basis information */
 
@@ -1232,16 +1277,22 @@ int lpx_dual_ratio_test(LPX *lp, int len, int ind[], double val[],
 /* additional utility routines ---------------------------------------*/
 
 LPX *lpx_read_mps(char *fname);
-/* read in problem data using MPS format */
+/* read problem data in fixed MPS format */
 
 int lpx_write_mps(LPX *lp, char *fname);
-/* write problem data using MPS format */
+/* write problem data in fixed MPS format */
 
 int lpx_read_bas(LPX *lp, char *fname);
-/* read predefined basis using MPS format */
+/* read LP basis in fixed MPS format */
 
 int lpx_write_bas(LPX *lp, char *fname);
-/* write current basis using MPS format */
+/* write LP basis in fixed MPS format */
+
+LPX *lpx_read_freemps(char *fname);
+/* read problem data in free MPS format */
+
+int lpx_write_freemps(LPX *lp, char *fname);
+/* write problem data in free MPS format */
 
 int lpx_print_prob(LPX *lp, char *fname);
 /* write problem data in plain text format */
@@ -1258,15 +1309,24 @@ int lpx_print_mip(LPX *lp, char *fname);
 int lpx_print_sens_bnds(LPX *lp, char *fname);
 /* write bounds sensitivity information */
 
-LPX *lpx_read_lpt(char *fname);
-/* read LP/MIP problem using CPLEX LP format */
+LPX *lpx_read_cpxlp(char *fname);
+/* read problem data in CPLEX LP format */
 
-int lpx_write_lpt(LPX *lp, char *fname);
-/* write problem data using CPLEX LP format */
+int lpx_write_cpxlp(LPX *lp, char *fname);
+/* write problem data in CPLEX LP format */
+
+LPX *lpx_extract_prob(void *mpl);
+/* extract problem instance from MathProg model */
 
 LPX *lpx_read_model(char *model, char *data, char *output);
 /* read LP/MIP model written in GNU MathProg language */
 
+LPX *lpx_read_prob(char *fname);
+/* read problem data in GNU LP format */
+
+int lpx_write_prob(LPX *lp, char *fname);
+/* write problem data in GNU LP format */
+
 #endif
 
 /* eof */
