@@ -29,6 +29,7 @@
 #include <setjmp.h>
 #include <stdio.h>
 #include "glpavl.h"
+#include "glprng.h"
 
 #define enter_context         glp_mpl_enter_context
 #define print_context         glp_mpl_print_context
@@ -86,9 +87,13 @@
 #define variable_statement    glp_mpl_variable_statement
 #define constraint_statement  glp_mpl_constraint_statement
 #define objective_statement   glp_mpl_objective_statement
+#define solve_statement       glp_mpl_solve_statement
 #define check_statement       glp_mpl_check_statement
 #define display_statement     glp_mpl_display_statement
+#define printf_statement      glp_mpl_printf_statement
+#define for_statement         glp_mpl_for_statement
 #define end_statement         glp_mpl_end_statement
+#define simple_statement      glp_mpl_simple_statement
 #define model_section         glp_mpl_model_section
 
 #define create_slice          glp_mpl_create_slice
@@ -222,17 +227,25 @@
 #define clean_check           glp_mpl_clean_check
 #define execute_display       glp_mpl_execute_display
 #define clean_display         glp_mpl_clean_display
+#define execute_printf        glp_mpl_execute_printf
+#define clean_printf          glp_mpl_clean_printf
+#define execute_for           glp_mpl_execute_for
+#define clean_for             glp_mpl_clean_for
+#define execute_statement     glp_mpl_execute_statement
+#define clean_statement       glp_mpl_clean_statement
 
 #define alloc_content         glp_mpl_alloc_content
 #define generate_model        glp_mpl_generate_model
 #define build_problem         glp_mpl_build_problem
+#define postsolve_model       glp_mpl_postsolve_model
 #define clean_model           glp_mpl_clean_model
 #define open_input            glp_mpl_open_input
 #define read_char             glp_mpl_read_char
 #define close_input           glp_mpl_close_input
 #define open_output           glp_mpl_open_output
+#define write_char            glp_mpl_write_char
 #define write_text            glp_mpl_write_text
-#define close_output          glp_mpl_close_output
+#define flush_output          glp_mpl_flush_output
 #define error                 glp_mpl_error
 #define warning               glp_mpl_warning
 #define mpl_initialize        glp_mpl_initialize
@@ -250,6 +263,9 @@
 #define mpl_get_col_name      glp_mpl_get_col_name
 #define mpl_get_col_kind      glp_mpl_get_col_kind
 #define mpl_get_col_bnds      glp_mpl_get_col_bnds
+#define mpl_has_solve_stmt    glp_mpl_has_solve_stmt
+#define mpl_put_col_value     glp_mpl_put_col_value
+#define mpl_postsolve         glp_mpl_postsolve
 #define mpl_terminate         glp_mpl_terminate
 
 typedef struct MPL MPL;
@@ -283,6 +299,9 @@ typedef struct CODE CODE;
 typedef struct CHECK CHECK;
 typedef struct DISPLAY DISPLAY;
 typedef struct DISPLAY1 DISPLAY1;
+typedef struct PRINTF PRINTF;
+typedef struct PRINTF1 PRINTF1;
+typedef struct FOR FOR;
 typedef struct STATEMENT STATEMENT;
 typedef struct TUPLE SLICE;
 
@@ -298,19 +317,22 @@ typedef struct TUPLE SLICE;
 #define A_ELEMSET       106   /* elemental set */
 #define A_ELEMVAR       107   /* elemental variable */
 #define A_EXPRESSION    108   /* expression */
-#define A_FORMULA       109   /* formula */
-#define A_INDEX         110   /* dummy index */
-#define A_INTEGER       111   /* something integer */
-#define A_LOGICAL       112   /* something logical */
-#define A_MAXIMIZE      113   /* objective has to be maximized */
-#define A_MINIMIZE      114   /* objective has to be minimized */
-#define A_NONE          115   /* nothing */
-#define A_NUMERIC       116   /* something numeric */
-#define A_PARAMETER     117   /* model parameter */
-#define A_SET           118   /* model set */
-#define A_SYMBOLIC      119   /* something symbolic */
-#define A_TUPLE         120   /* n-tuple */
-#define A_VARIABLE      121   /* model variable */
+#define A_FOR           109   /* for statement */
+#define A_FORMULA       110   /* formula */
+#define A_INDEX         111   /* dummy index */
+#define A_INTEGER       112   /* something integer */
+#define A_LOGICAL       113   /* something logical */
+#define A_MAXIMIZE      114   /* objective has to be maximized */
+#define A_MINIMIZE      115   /* objective has to be minimized */
+#define A_NONE          116   /* nothing */
+#define A_NUMERIC       117   /* something numeric */
+#define A_PARAMETER     118   /* model parameter */
+#define A_PRINTF        119   /* printf statement */
+#define A_SET           120   /* model set */
+#define A_SOLVE         121   /* solve statement */
+#define A_SYMBOLIC      122   /* something symbolic */
+#define A_TUPLE         123   /* n-tuple */
+#define A_VARIABLE      124   /* model variable */
 
 #define MAX_LENGTH 100
 /* maximal length of any symbolic value (this includes symbolic names,
@@ -320,6 +342,9 @@ typedef struct TUPLE SLICE;
 #define CONTEXT_SIZE 60
 /* size of the context queue, in characters */
 
+#define OUTBUF_SIZE 1024
+/* size of the output buffer, in characters */
+
 struct MPL
 {     /* translator database */
       /*--------------------------------------------------------------*/
@@ -424,18 +449,22 @@ struct MPL
          node.type = A_VARIABLE  => node.link -> VARIABLE
          node.type = A_CONSTRANT => node.link -> CONSTRAINT */
       STATEMENT *model;
-      /* linked list of model statements */
+      /* linked list of model statements in the original order */
       int flag_x;
       /* if this flag is set, the current token being left parenthesis
          begins a slice that allows recognizing any undeclared symbolic
          names as dummy indices; this flag is automatically reset once
          the next token has been scanned */
       int as_within;
-      /* the warning "in understood as within" was issued */
+      /* the warning "in understood as within" has been issued */
       int as_in;
-      /* the warning "within understood as in" was issued */
+      /* the warning "within understood as in" has been issued */
       int as_binary;
-      /* the warning "logical understood as binary" was issued */
+      /* the warning "logical understood as binary" has been issued */
+#if 1 /* 01/VIII-2004 */
+      int flag_s;
+      /* if this flag is set, the solve statement has been parsed */
+#endif
       /*--------------------------------------------------------------*/
       /* common segment */
       DMP *strings;
@@ -461,9 +490,17 @@ struct MPL
       char *tup_buf; /* char tup_buf[255+1]; */
       /* working buffer used by the routine format_tuple */
       /*--------------------------------------------------------------*/
-      /* generating segment */
+      /* generating/postsolving segment */
+#if 1 /* 04/VIII-2004 */
+      RNG *rand;
+      /* pseudo-random number generator */
+#endif
+#if 1 /* 01/VIII-2004 */
+      int flag_p;
+      /* if this flag is set, the postsolving phase is in effect */
+#endif
       STATEMENT *stmt;
-      /* model statement currently executed */
+      /* model statement being currently executed */
       int m;
       /* number of rows in the problem, m >= 0 */
       int n;
@@ -483,11 +520,17 @@ struct MPL
       char *in_file;
       /* name of the input text file */
       FILE *out_fp;
-      /* stream assigned to the output text file used to write all the
-         output produced by display statements; NULL means the display
-         output should be sent to stdout via the routine print */
+      /* stream assigned to the output text file used to write all data
+         produced by display and printf statements; NULL means the data
+         should be sent to stdout via the routine print */
       char *out_file;
       /* name of the output text file */
+#if 1 /* 03/VIII-2004 */
+      char *out_buf; /* char out_buf[OUTBUF_SIZE] */
+      /* buffer to accumulate output data */
+      int out_cnt;
+      /* count of data bytes stored in the output buffer */
+#endif
       /*--------------------------------------------------------------*/
       /* solver interface segment */
       jmp_buf jump;
@@ -497,7 +540,7 @@ struct MPL
          0 - database is being or has been initialized
          1 - model section is being or has been read
          2 - data section is being or has been read
-         3 - model is being or has been generated
+         3 - model is being or has been generated/postsolved
          4 - model processing error has occurred */
       char *mod_file;
       /* name of the input text file, which contains model section */
@@ -683,15 +726,27 @@ CONSTRAINT *constraint_statement(MPL *mpl);
 CONSTRAINT *objective_statement(MPL *mpl);
 /* parse objective statement */
 
+void *solve_statement(MPL *mpl);
+/* parse solve statement */
+
 CHECK *check_statement(MPL *mpl);
 /* parse check statement */
 
 DISPLAY *display_statement(MPL *mpl);
 /* parse display statement */
 
+PRINTF *printf_statement(MPL *mpl);
+/* parse printf statement */
+
+FOR *for_statement(MPL *mpl);
+/* parse for statement */
+
 void end_statement(MPL *mpl);
 /* parse end statement */
 
+STATEMENT *simple_statement(MPL *mpl, int spec);
+/* parse simple statement */
+
 void model_section(MPL *mpl);
 /* parse model section */
 
@@ -1183,6 +1238,10 @@ struct ELEMVAR
       double temp;
       /* working quantity used in operations on linear forms; normally
          it contains floating-point zero */
+#if 1 /* 01/VIII-2004 */
+      double value;
+      /* value of this elemental variable provided by the solver */
+#endif
 };
 
 /**********************************************************************/
@@ -2053,6 +2112,33 @@ struct DISPLAY1
       /* the next entry for the same statement */
 };
 
+struct PRINTF
+{     /* printf statement */
+      DOMAIN *domain;
+      /* subscript domain; NULL means domain is not used */
+      CODE *fmt;
+      /* pseudo-code for computing format string */
+      PRINTF1 *list;
+      /* printf list; can be empty */
+};
+
+struct PRINTF1
+{     /* printf list entry */
+      CODE *code;
+      /* pseudo-code for computing value to be printed */
+      PRINTF1 *next;
+      /* the next entry for the same statement */
+};
+
+struct FOR
+{     /* for statement */
+      DOMAIN *domain;
+      /* subscript domain; cannot be NULL */
+      STATEMENT *list;
+      /* linked list of model statements within this for statement in
+         the original order */
+};
+
 struct STATEMENT
 {     /* model statement */
       int line;
@@ -2063,15 +2149,21 @@ struct STATEMENT
          A_PARAMETER  - parameter statement
          A_VARIABLE   - variable statement
          A_CONSTRAINT - constraint/objective statement
+         A_SOLVE      - solve statement
          A_CHECK      - check statement
-         A_DISPLAY    - display statement */
+         A_DISPLAY    - display statement
+         A_PRINTF     - printf statement
+         A_FOR        - for statement */
       union
       {  SET *set;
          PARAMETER *par;
          VARIABLE *var;
          CONSTRAINT *con;
+         void *slv; /* currently not used (set to NULL) */
          CHECK *chk;
          DISPLAY *dpy;
+         PRINTF *prt;
+         FOR *fur;
       } u;
       /* specific part of statement */
       STATEMENT *next;
@@ -2091,8 +2183,26 @@ void execute_display(MPL *mpl, DISPLAY *dpy);
 void clean_display(MPL *mpl, DISPLAY *dpy);
 /* clean display statement */
 
+void execute_printf(MPL *mpl, PRINTF *prt);
+/* execute printf statement */
+
+void clean_printf(MPL *mpl, PRINTF *prt);
+/* clean printf statement */
+
+void execute_for(MPL *mpl, FOR *fur);
+/* execute for statement */
+
+void clean_for(MPL *mpl, FOR *fur);
+/* clean for statement */
+
+void execute_statement(MPL *mpl, STATEMENT *stmt);
+/* execute specified model statement */
+
+void clean_statement(MPL *mpl, STATEMENT *stmt);
+/* clean specified model statement */
+
 /**********************************************************************/
-/* * *                      GENERATING MODEL                      * * */
+/* * *              GENERATING AND POSTSOLVING MODEL              * * */
 /**********************************************************************/
 
 void alloc_content(MPL *mpl);
@@ -2104,6 +2214,9 @@ void generate_model(MPL *mpl);
 void build_problem(MPL *mpl);
 /* build problem instance */
 
+void postsolve_model(MPL *mpl);
+/* postsolve model */
+
 void clean_model(MPL *mpl);
 /* clean model content */
 
@@ -2123,11 +2236,14 @@ void close_input(MPL *mpl);
 void open_output(MPL *mpl, char *file);
 /* open output text file */
 
+void write_char(MPL *mpl, int c);
+/* write next character to output text file */
+
 void write_text(MPL *mpl, char *fmt, ...);
 /* format and write text to output text file */
 
-void close_output(MPL *mpl);
-/* close output text file */
+void flush_output(MPL *mpl);
+/* finalize writing data to output text file */
 
 /**********************************************************************/
 /* * *                      SOLVER INTERFACE                      * * */
@@ -2198,6 +2314,15 @@ int mpl_get_col_kind(MPL *mpl, int j);
 int mpl_get_col_bnds(MPL *mpl, int j, double *lb, double *ub);
 /* obtain column bounds */
 
+int mpl_has_solve_stmt(MPL *mpl);
+/* check if model has solve statement */
+
+void mpl_put_col_value(MPL *mpl, int j, double val);
+/* store column value */
+
+int mpl_postsolve(MPL *mpl);
+/* postsolve model */
+
 void mpl_terminate(MPL *mpl);
 /* free all resources used by translator */
 
