@@ -1,7 +1,7 @@
-/* glpqmd.h */
+/* glpqmd.h (quotient minimum degree algorithm) */
 
 /*----------------------------------------------------------------------
--- This file is a part of GLPK (GNU Linear Programming Kit).
+-- This file is part of GLPK (GNU Linear Programming Kit).
 --
 -- THIS CODE IS THE RESULT OF TRANSLATION OF THE FORTRAN SUBROUTINES
 -- GENQMD, QMDRCH, QMDQT, QMDUPD, AND QMDMRG FROM THE BOOK:
@@ -13,9 +13,7 @@
 -- OF THE ORIGINAL FORTRAN SUBROUTINES: ALAN GEORGE AND JOSEPH LIU,
 -- UNIVERSITY OF WATERLOO, WATERLOO, ONTARIO, CANADA.
 --
--- The translation was made by Andrew Makhorin <mao@mai2.rcnet.ru>,
--- <mao@gnu.org>, Department for Applied Informatics, Moscow Aviation
--- Institute, Moscow, Russia.
+-- The translation was made by Andrew Makhorin <mao@mai2.rcnet.ru>.
 --
 -- GLPK is free software; you can redistribute it and/or modify it
 -- under the terms of the GNU General Public License as published by
@@ -36,11 +34,11 @@
 #ifndef _GLPQMD_H
 #define _GLPQMD_H
 
-#define genqmd                glp_genqmd
-#define qmdrch                glp_qmdrch
-#define qmdqt                 glp_qmdqt
-#define qmdupd                glp_qmdupd
-#define qmdmrg                glp_qmdmrg
+#define genqmd                glp_qmd_genqmd
+#define qmdrch                glp_qmd_qmdrch
+#define qmdqt                 glp_qmd_qmdqt
+#define qmdupd                glp_qmd_qmdupd
+#define qmdmrg                glp_qmd_qmdmrg
 
 extern void genqmd(int *neqns, int xadj[], int adjncy[], int perm[],
       int invp[], int deg[], int marker[], int rchset[], int nbrhd[],
