@@ -80,10 +80,6 @@ int lib_init_env(void)
       env->mem_count = 0;
       env->mem_cpeak = 0;
       for (k = 0; k < LIB_MAX_OPEN; k++) env->file_slot[k] = NULL;
-      env->rand_val[0] = -1;
-      for (k = 1; k <= 55; k++) env->rand_val[k] = 0;
-      env->next_val = env->rand_val;
-      lib_init_rand(0);
       /* initialization completed */
       return 0;
 }
@@ -358,6 +354,7 @@ void _insist(char *expr, char *file, int line)
 -- The routine umalloc returns a pointer to the allocated memory block.
 -- To free this block the routine ufree (not free!) should be used. */
 
+#ifndef _GLPLIB_HUGEMEM
 void *umalloc(int size)
 {     LIBENV *env = lib_env_ptr();
       LIBMEM *desc;
@@ -389,6 +386,15 @@ void *umalloc(int size)
          env->mem_cpeak = env->mem_count;
       return (void *)((char *)desc + size_of_desc);
 }
+#else
+void *umalloc(int size)
+{     void *ptr;
+      ptr = malloc(size);
+      if (ptr == NULL)
+         fault("umalloc: size = %d; malloc failed", size);
+      return ptr;
+}
+#endif
 
 /*----------------------------------------------------------------------
 -- ucalloc - allocate memory block.
@@ -411,6 +417,7 @@ void *umalloc(int size)
 -- The routine ucalloc returns a pointer to the allocated memory block.
 -- To free this block the routine ufree (not free!) should be used. */
 
+#ifndef _GLPLIB_HUGEMEM
 void *ucalloc(int nmemb, int size)
 {     if (nmemb < 1)
          fault("ucalloc: nmemb = %d; invalid parameter", nmemb);
@@ -421,6 +428,16 @@ void *ucalloc(int nmemb, int size)
             nmemb, size);
       return umalloc(nmemb * size);
 }
+#else
+void *ucalloc(int nmemb, int size)
+{     void *ptr;
+      ptr = calloc(nmemb, size);
+      if (ptr == NULL)
+         fault("ucalloc: nmemb = %d; size = %d; calloc failed", nmemb,
+            size);
+      return ptr;
+}
+#endif
 
 /*----------------------------------------------------------------------
 -- ufree - free memory block.
@@ -435,6 +452,7 @@ void *ucalloc(int nmemb, int size)
 -- The routine ufree frees the memory block pointed to by ptr and which
 -- was previuosly allocated by the routine umalloc or ucalloc. */
 
+#ifndef _GLPLIB_HUGEMEM
 void ufree(void *ptr)
 {     LIBENV *env = lib_env_ptr();
       LIBMEM *desc;
@@ -460,6 +478,12 @@ void ufree(void *ptr)
       free(desc);
       return;
 }
+#else
+void ufree(void *ptr)
+{     free(ptr);
+      return;
+}
+#endif
 
 /*----------------------------------------------------------------------
 -- ufopen - open file.
