@@ -71,6 +71,10 @@ LPX *lpx_create_prob(void)
       lp->n = 0;
       lp->row = ucalloc(1+m_max, sizeof(LPXROW *));
       lp->col = ucalloc(1+n_max, sizeof(LPXCOL *));
+#if 1 /* 15/VIII-2004 */
+      lp->r_tree = NULL;
+      lp->c_tree = NULL;
+#endif
       /* LP basis */
       lp->b_stat = LPX_B_UNDEF;
       lp->basis = ucalloc(1+m_max, sizeof(int));
@@ -255,6 +259,9 @@ int lpx_add_rows(LPX *lp, int nrs)
          lp->row[i] = row = dmp_get_atom(lp->row_pool);
          row->i = i;
          row->name = NULL;
+#if 1 /* 15/VIII-2004 */
+         row->node = NULL;
+#endif
          row->type = LPX_FR;
          row->lb = row->ub = 0.0;
          row->ptr = NULL;
@@ -326,6 +333,9 @@ int lpx_add_cols(LPX *lp, int ncs)
          lp->col[j] = col = dmp_get_atom(lp->col_pool);
          col->j = j;
          col->name = NULL;
+#if 1 /* 15/VIII-2004 */
+         col->node = NULL;
+#endif
          col->kind = LPX_CV;
          col->type = LPX_FX;
          col->lb = col->ub = 0.0;
@@ -371,6 +381,13 @@ void lpx_set_row_name(LPX *lp, int i, char *name)
       if (!(1 <= i && i <= lp->m))
          fault("lpx_set_row_name: i = %d; row number out of range", i);
       row = lp->row[i];
+#if 1 /* 15/VIII-2004 */
+      if (row->node != NULL)
+      {  insist(lp->r_tree != NULL);
+         avl_delete_node(lp->r_tree, row->node);
+         row->node = NULL;
+      }
+#endif
       if (name == NULL || name[0] == '\0')
       {  if (row->name != NULL)
          {  delete_str(row->name);
@@ -383,6 +400,13 @@ void lpx_set_row_name(LPX *lp, int i, char *name)
          if (row->name == NULL) row->name = create_str(lp->str_pool);
          set_str(row->name, name);
       }
+#if 1 /* 15/VIII-2004 */
+      if (lp->r_tree != NULL && row->name != NULL)
+      {  insist(row->node == NULL);
+         row->node = avl_insert_by_key(lp->r_tree, row->name);
+         row->node->link = row;
+      }
+#endif
       return;
 }
 
@@ -408,6 +432,13 @@ void lpx_set_col_name(LPX *lp, int j, char *name)
          fault("lpx_set_col_name: j = %d; column number out of range",
             j);
       col = lp->col[j];
+#if 1 /* 15/VIII-2004 */
+      if (col->node != NULL)
+      {  insist(lp->c_tree != NULL);
+         avl_delete_node(lp->c_tree, col->node);
+         col->node = NULL;
+      }
+#endif
       if (name == NULL || name[0] == '\0')
       {  if (col->name != NULL)
          {  delete_str(col->name);
@@ -420,6 +451,13 @@ void lpx_set_col_name(LPX *lp, int j, char *name)
          if (col->name == NULL) col->name = create_str(lp->str_pool);
          set_str(col->name, name);
       }
+#if 1 /* 15/VIII-2004 */
+      if (lp->c_tree != NULL && col->name != NULL)
+      {  insist(col->node == NULL);
+         col->node = avl_insert_by_key(lp->c_tree, col->name);
+         col->node->link = col;
+      }
+#endif
       return;
 }
 
@@ -922,6 +960,54 @@ void lpx_load_matrix(LPX *lp, int ne, int ia[], int ja[], double ar[])
       return;
 }
 
+/*----------------------------------------------------------------------
+-- lpx_order_matrix - order rows and columns of the constraint matrix.
+--
+-- *Synopsis*
+--
+-- #include "glplpx.h"
+-- void lpx_order_matrix(LPX *lp);
+--
+-- *Description*
+--
+-- The routine lpx_order_matrix rebuilds row and column linked lists of
+-- the constraint matrix of the specified problem object.
+--
+-- On exit the constraint matrix is not changed, however, elements in
+-- the row linked lists are ordered in ascending their column indices,
+-- and elements in the column linked are ordered in ascending their row
+-- indices. */
+
+void lpx_order_matrix(LPX *lp)
+{     LPXAIJ *aij;
+      int i, j;
+      /* rebuild row lists */
+      for (i = lp->m; i >= 1; i--)
+         lp->row[i]->ptr = NULL;
+      for (j = lp->n; j >= 1; j--)
+      {  for (aij = lp->col[j]->ptr; aij != NULL; aij = aij->c_next)
+         {  i = aij->row->i;
+            aij->r_prev = NULL;
+            aij->r_next = lp->row[i]->ptr;
+            if (aij->r_next != NULL) aij->r_next->r_prev = aij;
+            lp->row[i]->ptr = aij;
+         }
+      }
+      /* rebuild column lists */
+      for (j = lp->n; j >= 1; j--)
+         lp->col[j]->ptr = NULL;
+      for (i = lp->m; i >= 1; i--)
+      {  for (aij = lp->row[i]->ptr; aij != NULL; aij = aij->r_next)
+         {  j = aij->col->j;
+            aij->c_prev = NULL;
+            aij->c_next = lp->col[j]->ptr;
+            if (aij->c_next != NULL) aij->c_next->c_prev = aij;
+            lp->col[j]->ptr = aij;
+         }
+      }
+      return;
+}
+
 /*----------------------------------------------------------------------
 -- lpx_set_rii - set (change) row scale factor.
 --
@@ -1118,6 +1204,9 @@ void lpx_del_rows(LPX *lp, int nrs, int num[])
                "t allowed", k, i);
          /* erase symbolic name assigned to the row */
          lpx_set_row_name(lp, i, NULL);
+#if 1 /* 15/VIII-2004 */
+         insist(row->node == NULL);
+#endif
          /* erase corresponding row of the constraint matrix */
          lpx_set_mat_row(lp, i, 0, NULL, NULL);
          /* mark the row to delete it */
@@ -1190,6 +1279,9 @@ void lpx_del_cols(LPX *lp, int ncs, int num[])
                " not allowed", k, j);
          /* erase symbolic name assigned to the column */
          lpx_set_col_name(lp, j, NULL);
+#if 1 /* 15/VIII-2004 */
+         insist(col->node == NULL);
+#endif
          /* erase corresponding column of the constraint matrix */
          lpx_set_mat_col(lp, j, 0, NULL, NULL);
          /* mark the column to delete it */
@@ -1244,12 +1336,102 @@ void lpx_delete_prob(LPX *lp)
       ufree(lp->str_buf);
       ufree(lp->row);
       ufree(lp->col);
+      if (lp->r_tree != NULL) avl_delete_tree(lp->r_tree);
+      if (lp->c_tree != NULL) avl_delete_tree(lp->c_tree);
       ufree(lp->basis);
       if (lp->b_inv != NULL) inv_delete(lp->b_inv);
       ufree(lp);
       return;
 }
 
+#if 1 /* 15/VIII-2004 */
+static int compare_names(void *info, void *key1, void *key2)
+{     insist(info == NULL);
+      return compare_str(key1, key2);
+}
+
+void lpx_create_index(LPX *lp)
+{     if (lp->r_tree == NULL)
+      {  LPXROW *row;
+         int i;
+         lp->r_tree = avl_create_tree(NULL, compare_names);
+         for (i = 1; i <= lp->m; i++)
+         {  row = lp->row[i];
+            insist(row->node == NULL);
+            if (row->name != NULL)
+            {  row->node = avl_insert_by_key(lp->r_tree, row->name);
+               row->node->link = row;
+            }
+         }
+      }
+      if (lp->c_tree == NULL)
+      {  LPXCOL *col;
+         int j;
+         lp->c_tree = avl_create_tree(NULL, compare_names);
+         for (j = 1; j <= lp->n; j++)
+         {  col = lp->col[j];
+            insist(col->node == NULL);
+            if (col->name != NULL)
+            {  col->node = avl_insert_by_key(lp->c_tree, col->name);
+               col->node->link = col;
+            }
+         }
+      }
+      return;
+}
+
+int lpx_find_row(LPX *lp, char *name)
+{     int i;
+      if (lp->r_tree == NULL)
+         fault("lpx_find_row: row index does not exist");
+      if (name == NULL || name[0] == '\0' || strlen(name) > 255)
+         i = 0;
+      else
+      {  AVLNODE *node;
+         STR *key;
+         key = create_str(lp->str_pool);
+         set_str(key, name);
+         node = avl_find_by_key(lp->r_tree, key);
+         delete_str(key);
+         i = (node == NULL ? 0 : ((LPXROW *)node->link)->i);
+      }
+      return i;
+}
+
+int lpx_find_col(LPX *lp, char *name)
+{     int j;
+      if (lp->c_tree == NULL)
+         fault("lpx_find_col: column index does not exist");
+      if (name == NULL || name[0] == '\0' || strlen(name) > 255)
+         j = 0;
+      else
+      {  AVLNODE *node;
+         STR *key;
+         key = create_str(lp->str_pool);
+         set_str(key, name);
+         node = avl_find_by_key(lp->c_tree, key);
+         delete_str(key);
+         j = (node == NULL ? 0 : ((LPXCOL *)node->link)->j);
+      }
+      return j;
+}
+
+void lpx_delete_index(LPX *lp)
+{     if (lp->r_tree != NULL)
+      {  int i;
+         for (i = 1; i <= lp->m; i++) lp->row[i]->node = NULL;
+         avl_delete_tree(lp->r_tree), lp->r_tree = NULL;
+      }
+      if (lp->c_tree != NULL)
+      {  int j;
+         for (j = 1; j <= lp->n; j++) lp->col[j]->node = NULL;
+         avl_delete_tree(lp->c_tree), lp->c_tree = NULL;
+      }
+      return;
+}
+#endif
+
+
 /*----------------------------------------------------------------------
 -- lpx_put_lp_basis - store LP basis information.
 --
