@@ -494,8 +494,8 @@ void matrix_format
 --
 -- This routine reads set data using the syntax:
 --
--- <set data> ::= set <set name> <assignments>
--- <set data> ::= set <set name> [ <symbol list> ] <assignments>
+-- <set data> ::= set <set name> <assignments> ;
+-- <set data> ::= set <set name> [ <symbol list> ] <assignments> ;
 -- <set name> ::= <symbolic name>
 -- <assignments> ::= <empty>
 -- <assignments> ::= <assignments> , :=
@@ -620,6 +620,7 @@ err2:          error(mpl, "transpose indicator (tr) incomplete");
          }
          else if (mpl->token == T_SEMICOLON)
          {  /* semicolon terminates the data block */
+            get_token(mpl /* ; */);
             break;
          }
          else
@@ -1042,9 +1043,9 @@ void tabbing_format
 --
 -- This routine reads parameter data using the syntax:
 --
--- <parameter data> ::= param <default value> : <tabbing format>
+-- <parameter data> ::= param <default value> : <tabbing format> ;
 -- <parameter data> ::= param <parameter name> <default value>
---                      <assignments>
+--                      <assignments> ;
 -- <parameter name> ::= <symbolic name>
 -- <default value> ::= <empty>
 -- <default value> ::= default <symbol>
@@ -1091,6 +1092,7 @@ void parameter_data(MPL *mpl)
          if (mpl->token != T_SEMICOLON)
             error(mpl, "symbol, number, or semicolon missing where expe"
                "cted");
+         get_token(mpl /* ; */);
          goto done;
       }
       /* in other cases there must be symbolic name of parameter, which
@@ -1164,6 +1166,7 @@ err3:          error(mpl, "transpose indicator (tr) incomplete");
          }
          else if (mpl->token == T_SEMICOLON)
          {  /* semicolon terminates the data block */
+            get_token(mpl /* ; */);
             break;
          }
          else
@@ -1193,28 +1196,8 @@ void data_section(MPL *mpl)
             set_data(mpl);
          else if (is_literal(mpl, "param"))
             parameter_data(mpl);
-         else if (is_literal(mpl, "check"))
-         {  warning(mpl, "check statement not allowed in data section; "
-               "statement ignored");
-            while (mpl->token != T_SEMICOLON)
-            {  if (mpl->token == T_EOF)
-                  error(mpl, "semicolon missing where expected");
-               get_token(mpl);
-            }
-         }
-         else if (is_literal(mpl, "display"))
-         {  warning(mpl, "display statement not allowed in data section"
-               "; statement ignored");
-            while (mpl->token != T_SEMICOLON)
-            {  if (mpl->token == T_EOF)
-                  error(mpl, "semicolon missing where expected");
-               get_token(mpl);
-            }
-         }
          else
             error(mpl, "syntax error in data section");
-         insist(mpl->token == T_SEMICOLON);
-         get_token(mpl /* ; */);
       }
       return;
 }
