@@ -26,362 +26,14 @@
 #include <ctype.h>
 #include <errno.h>
 #include <float.h>
+#include <limits.h>
 #include <stdarg.h>
 #include <string.h>
 #include "glplib.h"
 #include "glpmpl.h"
 
 /**********************************************************************/
-/* * *                      MODEL STATEMENTS                      * * */
-/**********************************************************************/
-
-/*----------------------------------------------------------------------
--- execute_check - execute check statement.
---
--- This routine executes specified check statement. */
-
-static int check_func(MPL *mpl, void *info)
-{     /* this is auxiliary routine to work within domain scope */
-      CHECK *chk = (CHECK *)info;
-      if (!eval_logical(mpl, chk->code))
-         error(mpl, "check%s failed", format_tuple(mpl, '[',
-            get_domain_tuple(mpl, chk->domain)));
-      return 0;
-}
-
-void execute_check(MPL *mpl, CHECK *chk)
-{     loop_within_domain(mpl, chk->domain, chk, check_func);
-      return;
-}
-
-/*----------------------------------------------------------------------
--- execute_display - execute display statement.
---
--- This routine executes specified display statement. */
-
-static void display_set(MPL *mpl, SET *set, MEMBER *memb)
-{     /* display member of model set */
-      ELEMSET *s = memb->value.set;
-      MEMBER *m;
-      write_text(mpl, "%s%s%s", set->name,
-         format_tuple(mpl, '[', memb->tuple),
-         s->head == NULL ? " is empty" : ":");
-      for (m = s->head; m != NULL; m = m->next)
-         write_text(mpl, "   %s", format_tuple(mpl, '(', m->tuple));
-      return;
-}
-
-static void display_par(MPL *mpl, PARAMETER *par, MEMBER *memb)
-{     /* display member of model parameter */
-      switch (par->type)
-      {  case A_NUMERIC:
-         case A_INTEGER:
-         case A_BINARY:
-            write_text(mpl, "%s%s = %.*g", par->name,
-               format_tuple(mpl, '[', memb->tuple),
-               DBL_DIG, memb->value.num);
-            break;
-         case A_SYMBOLIC:
-            write_text(mpl, "%s%s = %s", par->name,
-               format_tuple(mpl, '[', memb->tuple),
-               format_symbol(mpl, memb->value.sym));
-            break;
-         default:
-            insist(par != par);
-      }
-      return;
-}
-
-static void display_var(MPL *mpl, VARIABLE *var, MEMBER *memb)
-{     /* display member of model variable */
-      if (var->lbnd == NULL && var->ubnd == NULL)
-         write_text(mpl, "%s%s",
-            var->name, format_tuple(mpl, '[', memb->tuple));
-      else if (var->ubnd == NULL)
-         write_text(mpl, "%s%s >= %.*g",
-            var->name, format_tuple(mpl, '[', memb->tuple),
-            DBL_DIG, memb->value.var->lbnd);
-      else if (var->lbnd == NULL)
-         write_text(mpl, "%s%s <= %.*g",
-            var->name, format_tuple(mpl, '[', memb->tuple),
-            DBL_DIG, memb->value.var->ubnd);
-      else if (var->lbnd == var->ubnd)
-         write_text(mpl, "%s%s = %.*g",
-            var->name, format_tuple(mpl, '[', memb->tuple),
-            DBL_DIG, memb->value.var->lbnd);
-      else
-         write_text(mpl, "%.*g <= %s%s <= %.*g",
-            DBL_DIG, memb->value.var->lbnd,
-            var->name, format_tuple(mpl, '[', memb->tuple),
-            DBL_DIG, memb->value.var->ubnd);
-      return;
-}
-
-static void display_con(MPL *mpl, CONSTRAINT *con, MEMBER *memb)
-{     /* display member of model constraint */
-      FORMULA *term;
-      if (con->lbnd == NULL && con->ubnd == NULL)
-         write_text(mpl, "%s%s:",
-            con->name, format_tuple(mpl, '[', memb->tuple));
-      else if (con->ubnd == NULL)
-         write_text(mpl, "%s%s >= %.*g:",
-            con->name, format_tuple(mpl, '[', memb->tuple),
-            DBL_DIG, memb->value.con->lbnd);
-      else if (con->lbnd == NULL)
-         write_text(mpl, "%s%s <= %.*g:",
-            con->name, format_tuple(mpl, '[', memb->tuple),
-            DBL_DIG, memb->value.con->ubnd);
-      else if (con->lbnd == con->ubnd)
-         write_text(mpl, "%s%s = %.*g:",
-            con->name, format_tuple(mpl, '[', memb->tuple),
-            DBL_DIG, memb->value.con->lbnd);
-      else
-         write_text(mpl, "%.*g <= %s%s <= %.*g:",
-            DBL_DIG, memb->value.con->lbnd,
-            con->name, format_tuple(mpl, '[', memb->tuple),
-            DBL_DIG, memb->value.con->ubnd);
-      if ((con->type == A_MINIMIZE || con->type == A_MAXIMIZE) &&
-         memb->value.con->lbnd != 0)
-         write_text(mpl, "   %.*g", DBL_DIG, - memb->value.con->lbnd);
-      else if (memb->value.con->form == NULL)
-         write_text(mpl, "   empty linear form");
-      for (term = memb->value.con->form; term != NULL; term =
-         term->next)
-      {  insist(term->var != NULL);
-         write_text(mpl, "   %.*g %s%s", DBL_DIG, term->coef,
-            term->var->var->name, format_tuple(mpl, '[',
-            term->var->memb->tuple));
-      }
-      return;
-}
-
-static void display_memb(MPL *mpl, CODE *code)
-{     /* display member specified by pseudo-code */
-      MEMBER memb;
-      ARG_LIST *e;
-      insist(code->op == O_MEMNUM || code->op == O_MEMSYM ||
-             code->op == O_MEMSET || code->op == O_MEMVAR);
-      memb.tuple = create_tuple(mpl);
-      for (e = code->arg.par.list; e != NULL; e = e->next)
-         memb.tuple = expand_tuple(mpl, memb.tuple, eval_symbolic(mpl,
-            e->x));
-      switch (code->op)
-      {  case O_MEMNUM:
-            memb.value.num = eval_member_num(mpl, code->arg.par.par,
-               memb.tuple);
-            display_par(mpl, code->arg.par.par, &memb);
-            break;
-         case O_MEMSYM:
-            memb.value.sym = eval_member_sym(mpl, code->arg.par.par,
-               memb.tuple);
-            display_par(mpl, code->arg.par.par, &memb);
-            delete_symbol(mpl, memb.value.sym);
-            break;
-         case O_MEMSET:
-            memb.value.set = eval_member_set(mpl, code->arg.set.set,
-               memb.tuple);
-            display_set(mpl, code->arg.set.set, &memb);
-            break;
-         case O_MEMVAR:
-            memb.value.var = eval_member_var(mpl, code->arg.var.var,
-               memb.tuple);
-            display_var(mpl, code->arg.var.var, &memb);
-            break;
-         default:
-            insist(code != code);
-      }
-      delete_tuple(mpl, memb.tuple);
-      return;
-}
-
-static void display_code(MPL *mpl, CODE *code)
-{     /* display value of expression */
-      switch (code->type)
-      {  case A_NUMERIC:
-            /* numeric value */
-            {  double num;
-               num = eval_numeric(mpl, code);
-               write_text(mpl, "%.*g", DBL_DIG, num);
-            }
-            break;
-         case A_SYMBOLIC:
-            /* symbolic value */
-            {  SYMBOL *sym;
-               sym = eval_symbolic(mpl, code);
-               write_text(mpl, "%s", format_symbol(mpl, sym));
-               delete_symbol(mpl, sym);
-            }
-            break;
-         case A_LOGICAL:
-            /* logical value */
-            {  int bit;
-               bit = eval_logical(mpl, code);
-               write_text(mpl, "%s", bit ? "true" : "false");
-            }
-            break;
-         case A_TUPLE:
-            /* n-tuple */
-            {  TUPLE *tuple;
-               tuple = eval_tuple(mpl, code);
-               write_text(mpl, "%s", format_tuple(mpl, '(', tuple));
-               delete_tuple(mpl, tuple);
-            }
-            break;
-         case A_ELEMSET:
-            /* elemental set */
-            {  ELEMSET *set;
-               MEMBER *memb;
-               set = eval_elemset(mpl, code);
-               if (set->head == 0)
-                  write_text(mpl, "set is empty");
-               for (memb = set->head; memb != NULL; memb = memb->next)
-                  write_text(mpl, "   %s", format_tuple(mpl, '(',
-                     memb->tuple));
-               delete_elemset(mpl, set);
-            }
-            break;
-         case A_FORMULA:
-            /* linear form */
-            {  FORMULA *form, *term;
-               form = eval_formula(mpl, code);
-               if (form == NULL)
-                  write_text(mpl, "linear form is empty");
-               for (term = form; term != NULL; term = term->next)
-               {  if (term->var == NULL)
-                     write_text(mpl, "   %.*g", term->coef);
-                  else
-                     write_text(mpl, "   %.*g %s%s", DBL_DIG,
-                        term->coef, term->var->var->name,
-                        format_tuple(mpl, '[', term->var->memb->tuple));
-               }
-               delete_formula(mpl, form);
-            }
-            break;
-         default:
-            insist(code != code);
-      }
-      return;
-}
-
-static int display_func(MPL *mpl, void *info)
-{     /* this is auxiliary routine to work within domain scope */
-      DISPLAY *dpy = (DISPLAY *)info;
-      DISPLAY1 *entry;
-      for (entry = dpy->list; entry != NULL; entry = entry->next)
-      {  if (entry->type == A_INDEX)
-         {  /* dummy index */
-            DOMAIN_SLOT *slot = entry->u.slot;
-            write_text(mpl, "%s = %s", slot->name,
-            format_symbol(mpl, slot->value));
-         }
-         else if (entry->type == A_SET)
-         {  /* model set */
-            SET *set = entry->u.set;
-            MEMBER *memb;
-            if (set->assign != NULL)
-            {  /* the set has assignment expression; evaluate all its
-                  members over entire domain */
-               eval_whole_set(mpl, set);
-            }
-            else
-            {  /* the set has no assignment expression; refer to its
-                  any existing member ignoring resultant value to check
-                  the data provided the data section */
-               if (set->array->head != NULL)
-                  eval_member_set(mpl, set, set->array->head->tuple);
-            }
-            /* display all members of the set array */
-            if (set->array->head == NULL)
-               write_text(mpl, "%s has empty content", set->name);
-            for (memb = set->array->head; memb != NULL; memb =
-               memb->next) display_set(mpl, set, memb);
-         }
-         else if (entry->type == A_PARAMETER)
-         {  /* model parameter */
-            PARAMETER *par = entry->u.par;
-            MEMBER *memb;
-            if (par->assign != NULL)
-            {  /* the parameter has an assignment expression; evaluate
-                  all its member over entire domain */
-               eval_whole_par(mpl, par);
-            }
-            else
-            {  /* the parameter has no assignment expression; refer to
-                  its any existing member ignoring resultant value to
-                  check the data provided in the data section */
-               if (par->array->head != NULL)
-               {  if (par->type != A_SYMBOLIC)
-                     eval_member_num(mpl, par, par->array->head->tuple);
-                  else
-                     delete_symbol(mpl, eval_member_sym(mpl, par,
-                        par->array->head->tuple));
-               }
-            }
-            /* display all members of the parameter array */
-            if (par->array->head == NULL)
-               write_text(mpl, "%s has empty content", par->name);
-            for (memb = par->array->head; memb != NULL; memb =
-               memb->next) display_par(mpl, par, memb);
-         }
-         else if (entry->type == A_VARIABLE)
-         {  /* model variable */
-            VARIABLE *var = entry->u.var;
-            MEMBER *memb;
-            /* display all members of the variable array */
-            if (var->array->head == NULL)
-               write_text(mpl, "%s has empty content", var->name);
-            for (memb = var->array->head; memb != NULL; memb =
-               memb->next) display_var(mpl, var, memb);
-         }
-         else if (entry->type == A_CONSTRAINT)
-         {  /* model constraint */
-            CONSTRAINT *con = entry->u.con;
-            if (entry->list == NULL)
-            {  /* display the whole constraint */
-               MEMBER *memb;
-               eval_whole_con(mpl, con);
-               if (con->array->head == NULL)
-                  write_text(mpl, "%s has empty content", con->name);
-               for (memb = con->array->head; memb != NULL; memb =
-                  memb->next) display_con(mpl, con, memb);
-            }
-            else
-            {  /* display the constraint member */
-               TUPLE *tuple;
-               ARG_LIST *e;
-               ELEMCON *c;
-               tuple = create_tuple(mpl);
-               for (e = entry->list; e != NULL; e = e->next)
-                  tuple = expand_tuple(mpl, tuple, eval_symbolic(mpl,
-                     e->x));
-               c = eval_member_con(mpl, con, tuple);
-               delete_tuple(mpl, tuple);
-               display_con(mpl, con, c->memb);
-            }
-         }
-         else if (entry->type == A_EXPRESSION)
-         {  /* expression */
-            CODE *code = entry->u.code;
-            if (code->op == O_MEMNUM || code->op == O_MEMSYM ||
-                code->op == O_MEMSET || code->op == O_MEMVAR)
-               display_memb(mpl, code);
-            else
-               display_code(mpl, code);
-         }
-         else
-            insist(entry != entry);
-      }
-      return 0;
-}
-
-void execute_display(MPL *mpl, DISPLAY *dpy)
-{     /* main display routine */
-      loop_within_domain(mpl, dpy->domain, dpy, display_func);
-      return;
-}
-
-/**********************************************************************/
-/* * *                      GENERATING MODEL                      * * */
+/* * *              GENERATING AND POSTSOLVING MODEL              * * */
 /**********************************************************************/
 
 /*----------------------------------------------------------------------
@@ -434,11 +86,12 @@ void alloc_content(MPL *mpl)
                stmt->u.con->array = create_array(mpl, A_ELEMCON,
                   stmt->u.con->dim);
                break;
+            case A_SOLVE:
             case A_CHECK:
-               /* check statement (has no content array) */
-               break;
             case A_DISPLAY:
-               /* display statement (has no content array) */
+            case A_PRINTF:
+            case A_FOR:
+               /* functional statements have no content array */
                break;
             default:
                insist(stmt != stmt);
@@ -450,45 +103,17 @@ void alloc_content(MPL *mpl)
 /*----------------------------------------------------------------------
 -- generate_model - generate model.
 --
--- This routine executes all model statements in the order they follow
--- in the model section. */
+-- This routine executes the model statements which precede the solve
+-- statement. */
 
 void generate_model(MPL *mpl)
 {     STATEMENT *stmt;
-      insist(mpl->stmt == NULL);
-      /* walk through all model statements */
+      insist(!mpl->flag_p);
       for (stmt = mpl->model; stmt != NULL; stmt = stmt->next)
-      {  /* set global pointer to the statement currently executed */
-         mpl->stmt = stmt;
-         switch (stmt->type)
-         {  case A_SET:
-            case A_PARAMETER:
-            case A_VARIABLE:
-               /* model sets, parameters, and variables are evaluated
-                  indirectly */
-               break;
-            case A_CONSTRAINT:
-               /* evaluate model constraint or objective */
-               print("Generating %s...", stmt->u.con->name);
-               eval_whole_con(mpl, stmt->u.con);
-               break;
-            case A_CHECK:
-               /* execute check statement */
-               execute_check(mpl, stmt->u.chk);
-               break;
-            case A_DISPLAY:
-               /* execute display statement */
-               if (mpl->out_fp != NULL) write_text(mpl, "");
-               write_text(mpl, "Display statement at line %d",
-                  stmt->line);
-               execute_display(mpl, stmt->u.dpy);
-               break;
-            default:
-               insist(stmt != stmt);
-         }
+      {  execute_statement(mpl, stmt);
+         if (mpl->stmt->type == A_SOLVE) break;
       }
-      mpl->stmt = NULL;
-      /* looks like generation has been successfully completed :+) */
+      mpl->stmt = stmt;
       return;
 }
 
@@ -576,11 +201,27 @@ void build_problem(MPL *mpl)
       return;
 }
 
+/*----------------------------------------------------------------------
+-- postsolve_model - postsolve model.
+--
+-- This routine executes the model statements which follow the solve
+-- statement. */
+
+void postsolve_model(MPL *mpl)
+{     STATEMENT *stmt;
+      insist(!mpl->flag_p);
+      mpl->flag_p = 1;
+      for (stmt = mpl->stmt; stmt != NULL; stmt = stmt->next)
+         execute_statement(mpl, stmt);
+      mpl->stmt = NULL;
+      return;
+}
+
 /*----------------------------------------------------------------------
 -- clean_model - clean model content.
 --
--- This routine cleans model content that assumes deleting all stuff
--- dynamically allocated during the generation phase.
+-- This routine cleans the model content that assumes deleting all stuff
+-- dynamically allocated on generating/postsolving phase.
 --
 -- Actually cleaning model content is not needed. This function is used
 -- mainly to be sure that there were no logical errors on using dynamic
@@ -591,28 +232,8 @@ void build_problem(MPL *mpl)
 
 void clean_model(MPL *mpl)
 {     STATEMENT *stmt;
-      /* check that the generation phase either has not been performed
-         or has been performed successfully */
-      insist(mpl->stmt == NULL);
-      /* walk through all model statements */
       for (stmt = mpl->model; stmt != NULL; stmt = stmt->next)
-      {  switch(stmt->type)
-         {  case A_SET:
-               clean_set(mpl, stmt->u.set); break;
-            case A_PARAMETER:
-               clean_parameter(mpl, stmt->u.par); break;
-            case A_VARIABLE:
-               clean_variable(mpl, stmt->u.var); break;
-            case A_CONSTRAINT:
-               clean_constraint(mpl, stmt->u.con); break;
-            case A_CHECK:
-               clean_check(mpl, stmt->u.chk); break;
-            case A_DISPLAY:
-               clean_display(mpl, stmt->u.dpy); break;
-            default:
-               insist(stmt != stmt);
-         }
-      }
+         clean_statement(mpl, stmt);
       /* check that all atoms have been returned to their pools */
       if (mpl->strings->count != 0)
          error(mpl, "internal logic error: %d string segment(s) were lo"
@@ -714,56 +335,89 @@ void close_input(MPL *mpl)
 /*----------------------------------------------------------------------
 -- open_output - open output text file.
 --
--- This routine opens the output text file used to write all the output
--- produced by display statements. */
+-- This routine opens the output text file for writing data produced by
+-- display and printf statements. */
 
 void open_output(MPL *mpl, char *file)
 {     insist(mpl->out_fp == NULL);
-      mpl->out_fp = ufopen(file, "w");
-      if (mpl->out_fp == NULL)
-         error(mpl, "unable to create %s - %s", file, strerror(errno));
-      mpl->out_file = file;
-      write_text(mpl, "Start of display output");
+      if (file == NULL)
+      {  file = "<stdout>";
+         mpl->out_fp = stdout;
+      }
+      else
+      {  mpl->out_fp = ufopen(file, "w");
+         if (mpl->out_fp == NULL)
+            error(mpl, "unable to create %s - %s", file,
+               strerror(errno));
+      }
+      mpl->out_file = umalloc(strlen(file)+1);
+      strcpy(mpl->out_file, file);
+      mpl->out_buf = umalloc(OUTBUF_SIZE);
+      mpl->out_cnt = 0;
+      return;
+}
+
+/*----------------------------------------------------------------------
+-- write_char - write next character to output text file.
+--
+-- This routine writes a next ASCII character to the output text file.
+-- Note that line buffering is used, i.e. characters are stored in the
+-- output buffer until the new-line control character is passed, which
+-- causes actual writing data from the buffer to the output file. */
+
+void write_char(MPL *mpl, int c)
+{     insist(mpl->out_fp != NULL);
+      insist(mpl->out_cnt < OUTBUF_SIZE);
+      if (c == '\n')
+      {  /* flush the output buffer */
+         mpl->out_buf[mpl->out_cnt] = '\0';
+         if (mpl->out_fp == stdout)
+            print("%s", mpl->out_buf);
+         else
+            fprintf(mpl->out_fp, "%s\n", mpl->out_buf);
+         mpl->out_cnt = 0;
+      }
+      else
+      {  /* store the character to the output buffer */
+         mpl->out_buf[mpl->out_cnt++] = (char)c;
+         if (mpl->out_cnt == OUTBUF_SIZE)
+            error(mpl, "write error on %s - output buffer overflow",
+               mpl->out_file);
+      }
       return;
 }
 
 /*----------------------------------------------------------------------
 -- write_text - format and write text to output text file.
 --
--- This routine formats and writes a text produced by display statement
--- to the output text, or, if the file is not open, to stdout using the
--- print routine. */
+-- This routine formats a text using the format control string and then
+-- writes this text to the output text file. */
 
 void write_text(MPL *mpl, char *fmt, ...)
 {     va_list arg;
-      char msg[4095+1];
+      char buf[OUTBUF_SIZE], *c;
       va_start(arg, fmt);
-      vsprintf(msg, fmt, arg);
-      insist(strlen(msg) < sizeof(msg));
+      vsprintf(buf, fmt, arg);
+      insist(strlen(buf) < sizeof(buf));
       va_end(arg);
-      if (mpl->out_fp == NULL)
-         print("%s", msg);
-      else
-         fprintf(mpl->out_fp, "%s\n", msg);
+      for (c = buf; *c != '\0'; c++) write_char(mpl, *c);
       return;
 }
 
 /*----------------------------------------------------------------------
--- close_output - close output text file.
+-- flush_output - finalize writing data to output text file.
 --
--- This routine closes the output text file. */
+-- This routine finalizes writing data to the output text file. */
 
-void close_output(MPL *mpl)
+void flush_output(MPL *mpl)
 {     insist(mpl->out_fp != NULL);
-      write_text(mpl, "");
-      write_text(mpl, "End of display output");
-      fflush(mpl->out_fp);
-      if (ferror(mpl->out_fp))
-         error(mpl, "write error on %s - %s", mpl->out_file,
-            strerror(errno));
-      ufclose(mpl->out_fp);
-      mpl->out_fp = NULL;
-      mpl->out_file = NULL;
+      if (mpl->out_cnt > 0) write_char(mpl, '\n');
+      if (mpl->out_fp != stdout)
+      {  fflush(mpl->out_fp);
+         if (ferror(mpl->out_fp))
+            error(mpl, "write error on %s - %s", mpl->out_file,
+               strerror(errno));
+      }
       return;
 }
 
@@ -794,7 +448,7 @@ void error(MPL *mpl, char *fmt, ...)
             print_context(mpl);
             break;
          case 3:
-            /* generation phase */
+            /* generation/postsolve phase */
             print("%s:%d: %s",
                mpl->mod_file == NULL ? "(unknown)" : mpl->mod_file,
                mpl->stmt == NULL ? 0 : mpl->stmt->line, msg);
@@ -829,7 +483,7 @@ void warning(MPL *mpl, char *fmt, ...)
                mpl->line, msg);
             break;
          case 3:
-            /* generation phase */
+            /* generation/postsolve phase */
             print("%s:%d: warning: %s",
                mpl->mod_file == NULL ? "(unknown)" : mpl->mod_file,
                mpl->stmt == NULL ? 0 : mpl->stmt->line, msg);
@@ -892,6 +546,7 @@ MPL *mpl_initialize(void)
       mpl->as_within = 0;
       mpl->as_in = 0;
       mpl->as_binary = 0;
+      mpl->flag_s = 0;
       /* common segment */
       mpl->strings = dmp_create_pool(sizeof(STRING));
       mpl->symbols = dmp_create_pool(sizeof(SYMBOL));
@@ -906,7 +561,9 @@ MPL *mpl_initialize(void)
       mpl->sym_buf[0] = '\0';
       mpl->tup_buf = ucalloc(255+1, sizeof(char));
       mpl->tup_buf[0] = '\0';
-      /* generating segment */
+      /* generating/postsolving segment */
+      mpl->rand = rng_create_rand();
+      mpl->flag_p = 0;
       mpl->stmt = NULL;
       mpl->m = 0;
       mpl->n = 0;
@@ -917,6 +574,8 @@ MPL *mpl_initialize(void)
       mpl->in_file = NULL;
       mpl->out_fp = NULL;
       mpl->out_file = NULL;
+      mpl->out_buf = NULL;
+      mpl->out_cnt = 0;
       /* solver interface segment */
       if (setjmp(mpl->jump)) insist(mpl != mpl);
       mpl->phase = 0;
@@ -1075,8 +734,9 @@ done: /* return to the calling program */
 --
 -- The routine mpl_generate generates the model using its description
 -- stored in the translator database. This phase means generating all
--- variables, constraints, and objectives, performing check and display
--- statements, and building the problem instance.
+-- variables, constraints, and objectives, executing check and display
+-- statements, which precede the solve statement (if it is presented),
+-- and building the problem instance.
 --
 -- The character string file specifies the name of output text file, to
 -- which output produced by display statements should be written. It is
@@ -1090,7 +750,7 @@ done: /* return to the calling program */
 --
 -- The routine mpl_generate returns one of the following codes:
 --
--- 3 - model has been successfully generated. In this case the callng
+-- 3 - model has been successfully generated. In this case the calling
 --     program may call other api routines to obtain components of the
 --     problem instance from the translator database.
 -- 4 - processing failed due to some errors. In this case the calling
@@ -1104,9 +764,9 @@ int mpl_generate(MPL *mpl, char *file)
       if (setjmp(mpl->jump)) goto done;
       /* generate model */
       mpl->phase = 3;
-      if (file != NULL) open_output(mpl, file);
+      open_output(mpl, file);
       generate_model(mpl);
-      if (file != NULL) close_output(mpl);
+      flush_output(mpl);
       /* build problem instance */
       build_problem(mpl);
       /* generation phase has been finished */
@@ -1554,6 +1214,89 @@ int mpl_get_col_bnds(MPL *mpl, int j, double *_lb, double *_ub)
       return type;
 }
 
+/*----------------------------------------------------------------------
+-- mpl_has_solve_stmt - check if model has solve statement.
+--
+-- *Synopsis*
+--
+-- #include "glpmpl.h"
+-- int mpl_has_solve_stmt(MPL *mpl);
+--
+-- *Returns*
+--
+-- If the model has the solve statement, the routine returns non-zero,
+-- otherwise zero is returned. */
+
+int mpl_has_solve_stmt(MPL *mpl)
+{     if (mpl->phase != 3)
+         fault("mpl_has_solve_stmt: invalid call sequence");
+      return mpl->flag_s;
+}
+
+/*----------------------------------------------------------------------
+-- mpl_put_col_value - store column value.
+--
+-- *Synopsis*
+--
+-- #include "glpmpl.h"
+-- void mpl_put_col_value(MPL *mpl, int j, double val);
+--
+-- *Description*
+--
+-- The routine mpl_put_col_value stores numeric value of j-th column
+-- into the translator database. It is assumed that the column value is
+-- provided by the solver. */
+
+void mpl_put_col_value(MPL *mpl, int j, double val)
+{     if (mpl->phase != 3)
+         fault("mpl_put_col_value: invalid call sequence");
+      if (!(1 <= j && j <= mpl->n))
+         fault("mpl_put_col_value: j = %d; column number out of range",
+            j);
+      mpl->col[j]->value = val;
+      return;
+}
+
+/*----------------------------------------------------------------------
+-- mpl_postsolve - postsolve model.
+--
+-- *Synopsis*
+--
+-- #include "glpmpl.h"
+-- int mpl_postsolve(MPL *mpl);
+--
+-- *Description*
+--
+-- The routine mpl_postsolve performs postsolving of the model using
+-- its description stored in the translator database. This phase means
+-- executing statements, which follow the solve statement.
+--
+-- If this routine is used, it should be called once after the routine
+-- mpl_generate and if the latter returned the code 3.
+--
+-- *Returns*
+--
+-- The routine mpl_postsolve returns one of the following codes:
+--
+-- 3 - model has been successfully postsolved.
+-- 4 - processing failed due to some errors. In this case the calling
+--     program should call the routine mpl_terminate to terminate model
+--     processing. */
+
+int mpl_postsolve(MPL *mpl)
+{     if (!(mpl->phase == 3 && !mpl->flag_p))
+         fault("mpl_postsolve: invalid call sequence");
+      /* set up error handler */
+      if (setjmp(mpl->jump)) goto done;
+      /* perform postsolving */
+      postsolve_model(mpl);
+      flush_output(mpl);
+      /* postsolving phase has been finished */
+      print("Model has been successfully processed");
+done: /* return to the calling program */
+      return mpl->phase;
+}
+
 /*----------------------------------------------------------------------
 -- mpl_terminate - free all resources used by translator.
 --
@@ -1606,10 +1349,14 @@ void mpl_terminate(MPL *mpl)
       dmp_delete_pool(mpl->elemcons);
       ufree(mpl->sym_buf);
       ufree(mpl->tup_buf);
+      rng_delete_rand(mpl->rand);
       if (mpl->row != NULL) ufree(mpl->row);
       if (mpl->col != NULL) ufree(mpl->col);
       if (mpl->in_fp != NULL) ufclose(mpl->in_fp);
-      if (mpl->out_fp != NULL) ufclose(mpl->out_fp);
+      if (mpl->out_fp != NULL && mpl->out_fp != stdout)
+         ufclose(mpl->out_fp);
+      if (mpl->out_file != NULL) ufree(mpl->out_file);
+      if (mpl->out_buf != NULL) ufree(mpl->out_buf);
       if (mpl->mod_file != NULL) ufree(mpl->mod_file);
       ufree(mpl->mpl_buf);
       ufree(mpl);
